function W = svm_l21(X,Y)
%
% L21 regularized SVM (for festure selection SVM) implemented according to
% this paper:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Multi-Class L-2,1 Norm Support Vector Machine
% Xiao Cai, et al. U. Texas Arlington
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Chenyang Zhang, Date: 2014.9.16



%load testl21.mat

liblinear_path = '/home/medialab/Desktop/liblinear-1.94/matlab/'; % set the path to liblinear path
addpath(liblinear_path);
X = X';

p = size(X,1);
n = size(X,2);
k = 20;


% initialize with t = 0
Wt = ones(p,k);
Dt = getD(Wt);
W1_t = sqrtm(Dt)*Wt;

not_converge = true;
% stopping criterion
e = 0.0001;

t = 0;

while(not_converge)
    last_Dt = Dt;
    t1 = tic;
    % Relax
    X1_t = sqrtm(Dt)\X;
    % liblinear solver
    model = train(sparse(Y), sparse(X1_t'),'-s 4 -q');
    Wt = model.w';
    % update Dt
    Dt = getD(Wt);
    % get loss
    % loss = getLoss(sqrtm(Dt)\Wt,X,Y);
    % get prediction results
    p = X'*Wt;
    [acc,plb] = getL(p,Y);
    pte = Xte*Wt;
    [accte,plbte] = getL(pte,Yte);
    dt = toc(t1);
    DD = sum(abs(Dt(:)-last_Dt(:)));   
    
    if(DD < e)
        fprintf('converge... exit\n');
        break;
    end
    fprintf('iteration %d: Change in D: %.4f train acc = %3.1f%% test acc = %3.1f%% used: %f secs\n',t,DD,acc*100,accte*100,dt);
    t = t+1;
end
W = sqrtm(Dt)\Wt;
end